﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Management;
using Microsoft.Win32;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the Basic Input/Output System.
    /// </summary>
    public static class BIOS
    {
        private static ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_BIOS");

        /// <summary>
        /// Return the name of the BIOS.
        /// </summary>
        public static string Name
        {
            get
            {
                string name = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    name = queryObj["Name"].ToString();
                }
                return name;
            }
        }

        /// <summary>
        /// Returns the version of the BIOS.
        /// </summary>
        public static string Version
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\BIOS", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("BIOSVersion").ToString();
            }
        }

        /// <summary>
        /// Returns the release date of the BIOS.
        /// </summary>
        public static string ReleaseDate
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\BIOS", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                string reldate = regKey.GetValue("BIOSReleaseDate").ToString();
                DateTime date = new DateTime();
                if (reldate != string.Empty)
                {
                    string[] split = reldate.Split('/');
                    date = new DateTime(Convert.ToInt32(split[2]), Convert.ToInt32(split[0]), Convert.ToInt32(split[1]));
                }
                return date.ToShortDateString();
            }
        }

        /// <summary>
        /// Returns the manufacturer of the BIOS.
        /// </summary>
        public static string Manufacturer
        {
            get
            {
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey("HARDWARE\\DESCRIPTION\\System\\BIOS", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                return regKey.GetValue("BIOSVendor").ToString();
            }
        }

        /// <summary>
        /// Returns the serial number of the BIOS.
        /// </summary>
        public static string SerialNumber
        {
            get
            {
                string serNum = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    serNum = queryObj["SerialNumber"].ToString();
                }
                return serNum;
            }
        }

        /// <summary>
        /// Returns the current language of the BIOS.
        /// </summary>
        public static string CurrentLanguage
        {
            get
            {
                string currLang = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    currLang = queryObj["CurrentLanguage"].ToString();
                }
                return currLang;
            }
        }

        /// <summary>
        /// Returns the number of installable languages in the BIOS.
        /// </summary>
        public static int InstallableLanguages
        {
            get
            {
                int installLangs = 0;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    installLangs = Convert.ToInt32(queryObj["InstallableLanguages"].ToString());
                }
                return installLangs;
            }
        }

        /// <summary>
        /// Returns all available languages in the BIOS.
        /// </summary>
        public static string[] Languages
        {
            get
            {
                string[] langs = null;
                foreach (ManagementObject queryObj in searcher.Get())
                {
                    langs = (string[])queryObj["ListOfLanguages"];
                }
                return langs;
            }
        }

        /// <summary>
        /// Returns, whether the current BIOS is the primary BIOS.
        /// </summary>
        public static bool IsPrimaryBIOS
        {
            get
            {
                bool isPrimBios = false;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    isPrimBios = Convert.ToBoolean(queryObj["PrimaryBIOS"].ToString());
                }
                return isPrimBios;
                
            }
        }

        /// <summary>
        /// Returns the SM BIOS version of the BIOS.
        /// </summary>
        public static string SMBIOSVersion
        {
            get
            {
                string smBiosVersion = null;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    smBiosVersion = queryObj["SMBIOSBIOSVersion"].ToString();
                }
                return smBiosVersion;
            }
        }

        /// <summary>
        /// Returns the software element id of the BIOS.
        /// </summary>
        public static string SoftwareElementID
        {
            get
            {
                string softElemId = null;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    softElemId = queryObj["SoftwareElementID"].ToString();
                }
                return softElemId;
            }
        }

        /// <summary>
        /// Returns the index of the target operating system.
        /// </summary>
        public static int TargetOS
        {
            get
            {
                int targOs = 0;
                foreach(ManagementObject queryObj in searcher.Get())
                {
                    targOs = Convert.ToInt32(queryObj["TargetOperatingSystem"].ToString());
                }
                return targOs;
            }
        }
    }
}
