﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Management;

namespace TweakAPI.Info.Drives
{
    /// <summary>
    /// Provides information about the connected logical drives.
    /// </summary>
    public struct LogicalDrive
    {
        /// <summary>
        /// Enumerates all logical drives which are connected with this computer.
        /// </summary>
        /// <returns></returns>
        public static LogicalDrive[] EnumerateDrives()
        {
            List<LogicalDrive> lds = new List<LogicalDrive>();
            foreach (DriveInfo d in DriveInfo.GetDrives())
            {
                if (d.IsReady)
                {
                    LogicalDrive ld = new LogicalDrive();
                    ld.Name = d.Name;
                    ld.AvailableFreeSpace = d.AvailableFreeSpace;
                    ld.Format = d.DriveFormat;
                    ld.IsReady = d.IsReady;
                    ld.RootDirectory = d.RootDirectory.FullName;
                    ld.TotalFreeSpace = d.TotalFreeSpace;
                    ld.TotalSize = d.TotalSize;

                    if (d.DriveType == DriveType.CDRom)
                    { ld.Type = "Optisches Laufwerk"; }
                    else if (d.DriveType == DriveType.Fixed)
                    { ld.Type = "Festplatte"; }
                    else if (d.DriveType == DriveType.Network)
                    { ld.Type = "Netzlaufwerk"; }
                    else if (d.DriveType == DriveType.NoRootDirectory)
                    { ld.Type = "Laufwerk ohne Stammverzeichnis"; }
                    else if (d.DriveType == DriveType.Ram)
                    { ld.Type = "RAM-Laufwerk"; }
                    else if (d.DriveType == DriveType.Removable)
                    { ld.Type = "Wechseldatenträger"; }
                    else if (d.DriveType == DriveType.Unknown)
                    { ld.Type = "Unbekannt"; }

                    ld.VolumeLabel = d.VolumeLabel;
                    lds.Add(ld);
                }
            }
            return lds.ToArray();
        }

        /// <summary>
        /// The format.
        /// </summary>
        public string Format { get; set; }
        /// <summary>
        /// The amount of available free space.
        /// </summary>
        public long AvailableFreeSpace { get; set; }
        /// <summary>
        /// The type.
        /// </summary>
        public string Type { get; set; }
        /// <summary>
        /// Information, whether it is ready.
        /// </summary>
        public bool IsReady { get; set; }
        /// <summary>
        /// The name.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// The root directory.
        /// </summary>
        public string RootDirectory { get; set; }
        /// <summary>
        /// The amount of total free space.
        /// </summary>
        public long TotalFreeSpace { get; set; }
        /// <summary>
        /// The total size.
        /// </summary>
        public long TotalSize { get; set; }
        /// <summary>
        /// The volume label.
        /// </summary>
        public string VolumeLabel { get; set; }
    }

    /// <summary>
    /// Provides information about the optical drive(s).
    /// </summary>
    public struct OpticalDrive
    {
        /// <summary>
        /// Returns an <see cref="OpticalDrive"/>-Array containing information about the optical drive(s).
        /// </summary>
        /// <returns></returns>
        public static OpticalDrive[] EnumerateDrives()
        {
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_CDROMDrive");
            List<OpticalDrive> ods = new List<OpticalDrive>();
            foreach (ManagementObject obj in searcher.Get())
            {
                OpticalDrive od = new OpticalDrive();
                try { od.Name = obj["Name"].ToString(); } catch { }
                try { od.Description = obj["Description"].ToString(); } catch { }
                try { od.Drive = obj["Drive"].ToString(); } catch { }
                try { od.ID = obj["DeviceID"].ToString();} catch { }
                try { od.Manufacturer = obj["Manufacturer"].ToString();} catch { }
                try { od.SerialNumber = obj["SerialNumber"].ToString();} catch { }
                try { od.Status = obj["Status"].ToString();} catch { }
                try { od.VolumeName = obj["VolumeName"].ToString();} catch { }
                try { od.VolumeSerialNumber = obj["VolumeSerialNumber"].ToString();} catch { }
                try { od.MediaLoaded = bool.Parse(obj["MediaLoaded"].ToString());} catch { }
                try { od.DefaultBlockSize = int.Parse(obj["DefaultBlockSize"].ToString());} catch { }
                try { od.MaxBlockSize = int.Parse(obj["MaxBlockSize"].ToString());} catch { }
                try { od.MinBlockSize = int.Parse(obj["MinBlockSize"].ToString());} catch { }
                try { od.MediaType = obj["MediaType"].ToString();} catch { }
                try { od.NeedsCleaning = bool.Parse(obj["NeedsCleaning"].ToString());} catch { }
                try { od.RevisionLevel = obj["RevisionLevel"].ToString(); } catch { }
                ods.Add(od);
            }
            return ods.ToArray();
        }

        /// <summary>
        /// The name.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// The description.
        /// </summary>
        public string Description { get; set; }
        /// <summary>
        /// The drive caption.
        /// </summary>
        public string Drive { get; set; }
        /// <summary>
        /// The ID.
        /// </summary>
        public string ID { get; set; }
        /// <summary>
        /// The manufacturer.
        /// </summary>
        public string Manufacturer { get; set; }
        /// <summary>
        /// The serial number.
        /// </summary>
        public string SerialNumber { get; set; }
        /// <summary>
        /// The status.
        /// </summary>
        public string Status { get; set; }
        /// <summary>
        /// The volume name.
        /// </summary>
        public string VolumeName { get; set; }
        /// <summary>
        /// The volume serial number.
        /// </summary>
        public string VolumeSerialNumber { get; set; }
        /// <summary>
        /// Information, whether a medium is loaded. 
        /// </summary>
        public bool MediaLoaded { get; set; }
        /// <summary>
        /// The default block size.
        /// </summary>
        public int DefaultBlockSize { get; set; }
        /// <summary>
        /// The maximum block size.
        /// </summary>
        public int MaxBlockSize { get; set; }
        /// <summary>
        /// The type of the media.
        /// </summary>
        public string MediaType { get; set; }
        /// <summary>
        /// The minimum block size.
        /// </summary>
        public int MinBlockSize { get; set; }
        /// <summary>
        /// Information, whether it needs cleanin.
        /// </summary>
        public bool NeedsCleaning { get; set; }
        /// <summary>
        /// The revision level.
        /// </summary>
        public string RevisionLevel { get; set; }
    }

    /// <summary>
    /// Provides information about the connected physical drives.
    /// </summary>
    public struct PhysicalDrive
    {
        /// <summary>
        /// Returns an <see cref="PhysicalDrive"/>-Array containg information about the connected physical drives.
        /// </summary>
        /// <returns></returns>
        public static PhysicalDrive[] EnumerateDrives()
        {
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_DiskDrive");
            List<PhysicalDrive> pds = new List<PhysicalDrive>();
            foreach (ManagementObject queryObj in searcher.Get())
            {
                PhysicalDrive pd = new PhysicalDrive();
                try { pd.BytesPerSector = Convert.ToInt32(queryObj["BytesPerSector"].ToString()); } catch { }
                try { pd.Caption = queryObj["Caption"].ToString(); } catch { }
                try { pd.Cylinders = Convert.ToInt32(queryObj["TotalCylinders"].ToString()); } catch { }
                try { pd.FirmwareRevision = queryObj["FirmwareRevision"].ToString(); } catch { }
                try { pd.Heads = Convert.ToInt32(queryObj["TotalHeads"].ToString()); } catch { }
                try { pd.InterfaceType = queryObj["InterfaceType"].ToString(); } catch { }
                try { pd.MediaType = queryObj["MediaType"].ToString(); } catch { }
                try { pd.Name = queryObj["Name"].ToString(); } catch { }
                try { pd.Partitions = Convert.ToInt32(queryObj["Partitions"].ToString()); } catch { }
                try { pd.PNPDeviceID = queryObj["PNPDeviceID"].ToString(); } catch { }
                try { pd.Sectors = Convert.ToInt32(queryObj["TotalSectors"].ToString()); } catch { }
                try { pd.SectorsPerTrack = Convert.ToInt32(queryObj["SectorsPerTrack"].ToString()); } catch { }
                try { pd.SerialNumber = queryObj["SerialNumber"].ToString(); } catch { }
                try { pd.Size = Math.Round(((Convert.ToDecimal(queryObj["Size"].ToString()) / 1024) / 1024) / 1024, 2).ToString() + " GiB"; } catch { }
                try { pd.Tracks = Convert.ToInt32(queryObj["TotalTracks"].ToString()); } catch { }
                try { pd.TracksPerCylinder = Convert.ToInt32(queryObj["TracksPerCylinder"].ToString()); } catch { }
                pds.Add(pd);
            }
            return pds.ToArray();
        }
 
        /// <summary>
        /// The name.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// The caption.
        /// </summary>
        public string Caption { get; set; }
        /// <summary>
        /// The size.
        /// </summary>
        public string Size { get; set; }
        /// <summary>
        /// The interface type.
        /// </summary>
        public string InterfaceType { get; set; }
        /// <summary>
        /// The media type.
        /// </summary>
        public string MediaType { get; set; }
        /// <summary>
        /// The number of partitions.
        /// </summary>
        public int Partitions { get; set; }
        /// <summary>
        /// The number of cylinders.
        /// </summary>
        public int Cylinders { get; set; }
        /// <summary>
        /// The number of heads.
        /// </summary>
        public int Heads { get; set; }
        /// <summary>
        /// The number of sectors.
        /// </summary>
        public int Sectors { get; set; }
        /// <summary>
        /// The number of tracks.
        /// </summary>
        public int Tracks { get; set; }
        /// <summary>
        /// The number of tracks per cylinder.
        /// </summary>
        public int TracksPerCylinder { get; set; }
        /// <summary>
        /// The number of sectors per track.
        /// </summary>
        public int SectorsPerTrack { get; set; }
        /// <summary>
        /// The number of bytes per sector.
        /// </summary>
        public int BytesPerSector { get; set; }
        /// <summary>
        /// The firmware revision.
        /// </summary>
        public string FirmwareRevision { get; set; }
        /// <summary>
        /// The PNP device ID.
        /// </summary>
        public string PNPDeviceID { get; set; }
        /// <summary>
        /// The serial number.
        /// </summary>
        public string SerialNumber { get; set; }
    }
}
