﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Win32;
using System.Windows.Forms;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the entries in the startup.
    /// </summary>
    public struct StartupEntry
    {
        /// <summary>
        /// Returns a <see cref="StartupEntry"/>-Array containing information about the startup entries.
        /// </summary>
        /// <returns></returns>
        public static StartupEntry[] EnumerateEntries()
        {

            List<StartupEntry> entries = new List<StartupEntry>();
            RegistryKey regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadSubTree);
            try
            {
                foreach (string itm in regKey.GetValueNames())
                {
                    StartupEntry sEntry = new StartupEntry();
                    string value = regKey.GetValue(itm).ToString();
                    sEntry.Application = itm;
                    sEntry.Path = value;
                    sEntry.Validity = EntryValidity.AllUsersX64;
                    entries.Add(sEntry);
                }
            }
            catch { }

            RegistryKey RegKey2 = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadSubTree);
            try
            {
                foreach (string itm in RegKey2.GetValueNames())
                {
                    StartupEntry sEntry = new StartupEntry();
                    string value = RegKey2.GetValue(itm).ToString();
                    sEntry.Application = itm;
                    sEntry.Path = value;
                    sEntry.Validity = EntryValidity.AllUsersX86;
                    entries.Add(sEntry);
                }
            }
            catch { }

            RegistryKey RegKey3 = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadSubTree);
            try
            {
                foreach (string itm in RegKey3.GetValueNames())
                {
                    StartupEntry sEntry = new StartupEntry();
                    string value = RegKey3.GetValue(itm).ToString();
                    sEntry.Application = itm;
                    sEntry.Path = value;
                    sEntry.Validity = EntryValidity.CurrentUser;
                    entries.Add(sEntry);
                }
            }
            catch { }

            return entries.ToArray();
        }

        /// <summary>
        /// Creates a new startup entry.
        /// </summary>
        /// <param name="entry"></param>
        public static void CreateNewEntry(StartupEntry entry)
        {
            RegistryKey regKey = null;
            if (entry.Validity == EntryValidity.AllUsersX64)
            { regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadWriteSubTree); }
            else if (entry.Validity == EntryValidity.AllUsersX86)
            { regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadWriteSubTree); }
            else if (entry.Validity == EntryValidity.CurrentUser)
            { regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadWriteSubTree); }

            regKey.SetValue(entry.Application, entry.Path, RegistryValueKind.String);

            return;
        }

        /// <summary>
        /// Removes an existing startup entry.
        /// </summary>
        /// <param name="entry"></param>
        public static void RemoveEntry(StartupEntry entry)
        {
            RegistryKey regKey = null;
            if (entry.Validity == EntryValidity.AllUsersX64)
            { regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadWriteSubTree); }
            else if (entry.Validity == EntryValidity.AllUsersX86)
            { regKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadWriteSubTree); }
            else if (entry.Validity == EntryValidity.CurrentUser)
            { regKey = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Run", 
                RegistryKeyPermissionCheck.ReadWriteSubTree); }
            
            regKey.DeleteValue(entry.Application);
        }

        /// <summary>
        /// The name of the application.
        /// </summary>
        public string Application { get; set; }
        /// <summary>
        /// The path to the application.
        /// </summary>
        public string Path { get; set; }
        /// <summary>
        /// The validity of the startup entry.
        /// </summary>
        public EntryValidity Validity { get; set; }
    }

    /// <summary>
    /// Specifies the user validity for the startup entry.
    /// </summary>
    public enum EntryValidity
    {
        /// <summary>
        /// For all users (64 bit registry path).
        /// </summary>
        AllUsersX64 = 0,
        /// <summary>
        /// For all users (32 bit registry path).
        /// </summary>
        AllUsersX86 = 1,
        /// <summary>
        /// Only for the current user.
        /// </summary>
        CurrentUser = 2,
    }
}
